/*
 * Copyright (c) 2020, VeriSilicon Holdings Co., Ltd. All rights reserved
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __VPI_API_H__
#define __VPI_API_H__

/*
 * provides application programming interface for the application layer.
 */
#if defined(__cplusplus)
extern "C" {
#endif

#include "vpi_types.h"

//System
VpiHandle vpi_create(const char *app_name);
void vpi_destroy(VpiHandle);

//Device
VpiDevHandle vpi_device_create(VpiHandle handle, const char *device);
void vpi_device_destroy(VpiDevHandle handle);
VpiRet vpi_device_set_param(VpiDevHandle dev_ctx, char * key, char * value);
VpiRet vpi_device_open(VpiDevHandle dev_ctx);
void vpi_device_close(VpiDevHandle dev_ctx);
VpiHwCfg* vpi_device_get_hw_config(VpiDevHandle dev_ctx, VpiHwId hw_id);
void vpi_device_free_hw_config(VpiHwCfg *hw_cfg);
VpiRet vpi_device_get_device_index(VpiDevHandle handle, int* device_index);

//Memory
VpiMemHandle vpi_memory_init(VpiHandle handle, VpiDevHandle dev_ctx);
VpiRet vpi_memory_get_info(VpiMemHandle handle, VpiMemInfo *info);
VpiRet vpi_memory_set_map_info(VpiMemHandle handle, int pp_index, int out_index);
VpiRet vpi_memory_get_map_info(VpiMemHandle handle, int out_index, int *p_pp_index);
VpiRet vpi_memory_set_channel_info(VpiMemHandle handle, VpiChannelInfo *ch_info, VpiHwId hw_id);
VpiRet vpi_memory_set_single_channel_info(VpiMemHandle handle, int idx, VpiChannelInfo *ch_info, VpiHwId hw_id);
VpiRet vpi_memory_get_channel_info(VpiMemHandle handle, VpiChannelInfo *ch_info);
int vpi_info_get_struct_size(VpiStructType struct_type);
void *vpi_memory_get_struct(VpiStructType struct_type);
void vpi_memory_free_struct(void *p_struct);
void vpi_memory_release(VpiMemHandle handle);

//Picture/Frame
VpiRet vpi_pic_set_buffer(VpiFrm *in_pic, int ch, int plane, void *buffer, unsigned int linesize);
VpiRet vpi_frame_alloc(VpiMemHandle mem_ctx, VpiFrm **frame);
void vpi_frame_free(VpiMemHandle mem_ctx, VpiFrm *pic);
void vpi_frame_ref(VpiFrm *frame);
void vpi_frame_unref(VpiFrm *frame);
void vpi_frame_release(VpiFrm *frame);
void vpi_frame_remove_ref(VpiFrm *frame);

//Decoder
VpiDecHandle vpi_decoder_init(VpiDecInitParams *params);
VpiRet vpi_decoder_set_param(VpiDecHandle handle, const char * key, char * value);
VpiRet vpi_decoder_open(VpiDecHandle handle);
void vpi_decoder_close(VpiDecHandle handle);
int vpi_decoder_put_packet(VpiDecHandle handle, VpiPkt *pkt);
int vpi_decoder_get_frame(VpiDecHandle handle);
VpiRet vpi_decoder_get_output_frame(VpiDecHandle handle, VpiFrm *pic);
void *vpi_decoder_release_pkt(VpiDecHandle handle);
void *vpi_decoder_flush_pkt(void *pool);

//Encoder
VpiEncHandle vpi_encoder_init(VpiEncInitParams *param);
VpiRet vpi_encoder_set_param(VpiEncHandle handle, const char *key, char *value);
VpiRet vpi_encoder_open(VpiEncHandle handle);
VpiRet vpi_encoder_put_frame(VpiEncHandle handle, VpiFrm *pic);
VpiRet vpi_encoder_get_packet_size(VpiEncHandle handle, int *size);
VpiRet vpi_encoder_get_packet(VpiEncHandle handle, VpiPkt *pkt);
VpiRet vpi_encoder_get_reorder_delay(VpiEncHandle handle, unsigned int *reorder_delay);
void vpi_encoder_close(VpiEncHandle handle);
void *vpi_encoder_get_extradata(VpiEncHandle handle, int *extradata_size);
void *vpi_encoder_release_frame(VpiEncHandle handle);
void *vpi_encoder_flush_frame(void *pool);

//Processor
VpiRet vpi_processor_init(VpiProcInitParams *param, VpiProcHandle *handle);
VpiRet vpi_processor_set_param(VpiProcHandle handle, const char *key, char *value);
VpiRet vpi_processor_open(VpiProcHandle handle);
VpiRet vpi_processor_get_info(VpiProcHandle handle, VpiProcInfo *info);
VpiRet vpi_processor_process_frame(VpiProcHandle handle, VpiFrm * in, VpiFrm * out);
VpiRet vpi_processor_process_frame_list(VpiProcHandle handle, VpiFrm * in, VpiFrm ** out);
void vpi_processor_close(VpiProcHandle handle);

// Common
char *vpi_error_str(int vpi_error);
void vpi_insert_mem_node(void *ptr, unsigned int size, unsigned int mem_type);
void vpi_remove_mem_node(void *ptr, unsigned int size);
VpiRet vpi_pool_create(VpiMemHandle handle, unsigned int num, void **pool);
void vpi_pool_unref(void *pool);

#if defined(__cplusplus)
}
#endif

#endif
